@@ -1,22 +1,19 @@
 from __future__ import absolute_import
 
-from uuid import uuid4
-
 import numpy as np
 import pandas as pd
 import six
 from bson import Code
 from numpy import isscalar
-from pandas.io.json import json_normalize
 from pymongo.collection import Collection
+from uuid import uuid4
 
-from omegaml import defaults
 from omegaml.store import qops
 from omegaml.store.filtered import FilteredCollection
 from omegaml.store.query import Filter, MongoQ
 from omegaml.store.queryops import MongoQueryOps
 from omegaml.util import make_tuple, make_list, restore_index, \
-    cursor_to_dataframe, restore_index_columns_order, PickableCollection, extend_instance
+    cursor_to_dataframe, restore_index_columns_order, PickableCollection, extend_instance, json_normalize
 
 INSPECT_CACHE = []
 
@@ -59,11 +56,11 @@ class MGrouper(object):
         """
         aggregate by given specs
 
-        See the following link for a list of supported operations. 
+        See the following link for a list of supported operations.
         https://docs.mongodb.com/manual/reference/operator/aggregation/group/
 
-        :param specs: a dictionary of { column : function | list[functions] } 
-           pairs. 
+        :param specs: a dictionary of { column : function | list[functions] }
+           pairs.
         """
 
         def add_stats(specs, column, stat):
@@ -202,11 +199,11 @@ class MLocIndexer(object):
         if isinstance(specs, np.ndarray):
             specs = specs.tolist()
         if (isinstance(specs, enumerable_types)
-                and isscalar(specs[0]) and len(idx_cols) == 1
-                and not any(isinstance(s, slice) for s in specs)):
+              and isscalar(specs[0]) and len(idx_cols) == 1
+              and not any(isinstance(s, slice) for s in specs)):
             # single column index with list of scalar values
             if (self.positional and isinstance(specs, tuple) and len(specs) == 2
-                    and all(isscalar(v) for v in specs)):
+                  and all(isscalar(v) for v in specs)):
                 # iloc[int, int] is a cell access
                 flt_kwargs[idx_cols[0]] = specs[0]
                 projection.extend(self._get_projection(specs[1]))
@@ -304,7 +301,7 @@ class MSeriesGroupby(MGrouper):
         """
         return series count
 
-        :return: counts by group 
+        :return: counts by group
         """
         # MGrouper will insert a _count column, see _count(). we remove
         # that column again and return a series named as the group column
@@ -421,7 +418,7 @@ class MDataFrame(object):
         # FIXME inefficient. make MDataFrame a drop-in replacement for any numpy ndarray
         # this evaluates every single time
         if self._evaluated is None:
-            self._evaluated = array = self.value.as_matrix()
+            self._evaluated = array = self.value.values()
         else:
             array = self._evaluated
         return array
@@ -558,7 +555,7 @@ class MDataFrame(object):
         else:
             data = self._inspect_cache
         if not (raw or explain):
-            data = pd.DataFrame(pd.io.json.json_normalize(data))
+            data = pd.DataFrame(json_normalize(data))
         return data
 
     def count(self):
@@ -592,7 +589,7 @@ class MDataFrame(object):
         """
         resolve the query and return a Pandas DataFrame
 
-        :return: the result of the query as a pandas DataFrame 
+        :return: the result of the query as a pandas DataFrame
         """
         cursor = self._get_cursor()
         df = self._get_dataframe_from_cursor(cursor)
@@ -624,7 +621,7 @@ class MDataFrame(object):
         return self
 
     def _get_dataframe_from_cursor(self, cursor):
-        """ 
+        """
         from the given cursor return a DataFrame
         """
         df = cursor_to_dataframe(cursor, parser=self._parser)
@@ -694,7 +691,7 @@ class MDataFrame(object):
         skip the topn number of rows
 
         :param topn: the number of rows to skip.
-        :return: the MDataFrame 
+        :return: the MDataFrame
         """
         return self._clone(skip=topn)
 
@@ -709,13 +706,13 @@ class MDataFrame(object):
         :param right: the other MDataFrame
         :param on: the list of key columns to merge by
         :param left_on: the list of the key columns to merge on this dataframe
-        :param right_on: the list of the key columns to merge on the other 
+        :param right_on: the list of the key columns to merge on the other
            dataframe
-        :param how: the method to merge. supported are left, inner, right. 
+        :param how: the method to merge. supported are left, inner, right.
            Defaults to inner
         :param target: the name of the collection to store the merge results
            in. If not provided a temporary name will be created.
-        :param suffixes: the suffixes to apply to identical left and right 
+        :param suffixes: the suffixes to apply to identical left and right
            columns
         :param sort: if True the merge results will be sorted. If False the
            MongoDB natural order is implied.
@@ -742,7 +739,7 @@ class MDataFrame(object):
         target_name = self._get_collection_name_of(
             target, '_temp.merge.%s' % uuid4().hex)
         target_field = (
-                "%s_%s" % (right_name.replace('.', '_'), right_on or on))
+              "%s_%s" % (right_name.replace('.', '_'), right_on or on))
         lookup = qops.LOOKUP(right_name,
                              key=on,
                              left_key=left_on,
@@ -857,14 +854,14 @@ class MDataFrame(object):
         return name
 
     def _get_filter_criteria(self, *args, **kwargs):
-        """ 
+        """
         return mongo query from filter specs
 
         this uses a Filter to produce the query from the kwargs.
 
         :param args: a Q object or logical combination of Q objects
            (optional)
-        :param kwargs: all AND filter criteria 
+        :param kwargs: all AND filter criteria
         """
         if len(args) > 0:
             q = args[0]
@@ -878,14 +875,14 @@ class MDataFrame(object):
 
     def query_inplace(self, *args, **kwargs):
         """
-        filters this MDataFrame and returns it. 
+        filters this MDataFrame and returns it.
 
         Any subsequent operation on the dataframe will have the filter
         applied. To reset the filter call .reset() without arguments.
 
         :param args: a Q object or logical combination of Q objects
            (optional)
-        :param kwargs: all AND filter criteria 
+        :param kwargs: all AND filter criteria
         :return: self
         """
         self._evaluated = None
@@ -906,7 +903,7 @@ class MDataFrame(object):
 
         :param args: a Q object or logical combination of Q objects
            (optional)
-        :param kwargs: all AND filter criteria 
+        :param kwargs: all AND filter criteria
         :return: a new MDataFrame with the filter applied
         """
         effective_filter = dict(self.filter_criteria)
@@ -932,6 +929,48 @@ class MDataFrame(object):
         """
         return cursor_to_dataframe(self.collection.list_indexes())
 
+    def iterchunks(self, chunksize=100):
+        """
+        return an iterator
+
+        Args:
+            chunksize (int): number of rows in each chunk
+
+        Returns:
+            a dataframe of max. length chunksize
+        """
+        i = 0
+        while True:
+            chunkdf = self.skip(i).head(chunksize).value
+            if len(chunkdf) == 0:
+                break
+            i += chunksize
+            yield chunkdf
+
+    def itertuples(self, chunksize=1000):
+        __doc__ = pd.DataFrame.itertuples.__doc__
+
+        for chunkdf in self.iterchunks(chunksize=chunksize):
+            for row in chunkdf.iterrows():
+                yield row
+
+    def iterrows(self, chunksize=1000):
+        __doc__ = pd.DataFrame.iterrows.__doc__
+
+        for chunkdf in self.iterchunks(chunksize=chunksize):
+            for row in chunkdf.iterrows():
+                yield row
+
+    def iteritems(self):
+        __doc__ = pd.DataFrame.iteritems.__doc__
+        return self.items()
+
+    def items(self):
+        __doc__ = pd.DataFrame.items.__doc__
+
+        for col in self.columns:
+            yield col, self[col].value
+
     @property
     def loc(self):
         """
@@ -959,7 +998,7 @@ class MDataFrame(object):
 
 class MSeries(MDataFrame):
     """
-    Series implementation for MDataFrames 
+    Series implementation for MDataFrames
 
     behaves like a DataFrame but limited to one column.
     """
@@ -1008,7 +1047,7 @@ class MSeries(MDataFrame):
         only distinct values are returned as an array, matching
         the behavior of a Series
 
-        :return: pandas.Series 
+        :return: pandas.Series
         """
         cursor = self._get_cursor()
         column = make_tuple(self.columns)[0]
