@@ -1,16 +1,15 @@
 from __future__ import absolute_import
 
-import unittest
-import uuid
 from unittest import skip
-from zipfile import ZipFile
 
 import gridfs
+import joblib
 import pandas as pd
+import unittest
+import uuid
 from datetime import timedelta
 from mongoengine.connection import disconnect
 from mongoengine.errors import DoesNotExist, FieldDoesNotExist
-from pandas.io.json import json_normalize
 from pandas.util import testing
 from pandas.util.testing import assert_frame_equal, assert_series_equal
 from six import BytesIO
@@ -18,13 +17,13 @@ from six.moves import range
 from sklearn.datasets import load_iris
 from sklearn.linear_model import LogisticRegression
 
-from omegaml import backends
 from omegaml.backends.rawdict import PandasRawDictBackend
 from omegaml.backends.rawfiles import PythonRawFileBackend
+from omegaml.backends.scikitlearn import ScikitLearnBackend
 from omegaml.documents import MDREGISTRY
 from omegaml.mdataframe import MDataFrame
 from omegaml.store import OmegaStore
-from omegaml.util import delete_database
+from omegaml.util import delete_database, json_normalize
 
 
 class StoreTests(unittest.TestCase):
@@ -48,11 +47,13 @@ class StoreTests(unittest.TestCase):
         lr.fit(X, Y)
         result = lr.predict(X)
         # package locally
-        backend = backends.ScikitLearnBackend(model_store=store,
-                                              data_store=store)
-        zipfname = backend._package_model(lr, 'models/foo')
+        backend = ScikitLearnBackend(model_store=store,
+                                     data_store=store)
+        # v2 of the ScikitLearnBackend no longer supports testing these methods
+        # test put(), get() instead
+        zipfname = backend._v1_package_model(lr, 'models/foo')
         # load it, try predicting
-        lr2 = backend._extract_model(zipfname)
+        lr2 = backend._v1_extract_model(zipfname)
         self.assertIsInstance(lr2, LogisticRegression)
         result2 = lr2.predict(X)
         self.assertTrue((result == result2).all())
@@ -352,12 +353,11 @@ class StoreTests(unittest.TestCase):
         lr = LogisticRegression(solver='liblinear', multi_class='auto')
         lr.fit(X, Y)
         # store it remote
-        store.put(lr, 'foo')
+        store.put(lr, 'foox', _kind_version='1')
         # get it back as a zipfile
-        lr2file = store.get('foo', force_python=True)
-        contents = lr2file.read()
-        with ZipFile(BytesIO(contents)) as zipf:
-            self.assertIn('foo', zipf.namelist())
+        lr2file = store.get('foox', force_python=True)
+        lr_ = joblib.load(lr2file)
+        self.assertIsInstance(lr_, LogisticRegression)
 
     def test_store_with_metadata(self):
         om = OmegaStore(prefix='')
@@ -663,7 +663,6 @@ class StoreTests(unittest.TestCase):
         # currently this fails, see @skip reason
         testing.assert_frame_equal(df, df2)
 
-
     def test_store_dict_in_df(self):
         df = pd.DataFrame({
             'x': [{'foo': 'bar '}],
@@ -775,6 +774,26 @@ class StoreTests(unittest.TestCase):
         store.put('/tmp/testfile.txt', 'myfile')
         self.assertEqual(data.encode('utf-8'), store.get('myfile').read())
 
-
-
-
+    def test_bucket(self):
+        # test different buckets actually separate objects by the same name
+        # -- data
+        foo_store = OmegaStore(bucket='foo')
+        bar_store = OmegaStore(bucket='bar')
+        foo_store.register_backend(PythonRawFileBackend.KIND, PythonRawFileBackend)
+        bar_store.register_backend(PythonRawFileBackend.KIND, PythonRawFileBackend)
+        foo_data = {'foo': 'bar',
+                    'bax': 'fox'}
+        bar_data = {'foo': 'bax',
+                    'bax': 'foz'}
+        foo_store.put(foo_data, 'data')
+        bar_store.put(bar_data, 'data')
+        self.assertEqual(foo_store.get('data')[0], foo_data)
+        self.assertEqual(bar_store.get('data')[0], bar_data)
+        # -- files
+        foo_data = "some data"
+        file_like = BytesIO(foo_data.encode('utf-8'))
+        foo_store.put(file_like, 'myfile')
+        bar_data = "some other data"
+        file_like = BytesIO(bar_data.encode('utf-8'))
+        bar_store.put(file_like, 'myfile')
+        self.assertNotEqual(foo_store.get('myfile').read(), bar_store.get('myfile').read())
